#ifndef MSP430ADC12_H
#define MSP430ADC12_H

#define ADC12CTL0_DEFAULT 0x0000
#define ADC12CTL1_DEFAULT 0x0218

#define ASSOCIATE_ADC_CHANNEL(IC, RV, RVL) \
        ((((RVL << 3) + RV) << 4) + IC)
        
#define SET_ADC12_STANDARD_SETTINGS(IC, RV, SHT, RVL)   \
        ((MSP430ADC12StandardSettings_ut)((((((((uint16_t) RVL) << 4) + SHT) << 3) + RV) << 4) + IC))
 
#define SET_ADC12_ADVANCED_SETTINGS(IC, RV, SHT, CS, CD, SHS, RVL) \
            ((MSP430ADC12AdvancedSettings_ut) ((((((((((((((uint32_t) RVL) << 2) + SHS) << 3) + CD) << 2) + CS) << 4) + SHT) << 3) + RV) << 4) + IC))
      
typedef struct {
  unsigned int inputChannel: 4;
  unsigned int referenceVoltage: 3;
  unsigned int sampleHoldTime: 4;
  unsigned int refVolt2_5: 1;
} __attribute__ ((packed)) MSP430ADC12StandardSettings_t;

typedef union {
   uint16_t i;
   MSP430ADC12StandardSettings_t s;
} MSP430ADC12StandardSettings_ut;

typedef struct  {
  unsigned int inputChannel: 4;
  unsigned int referenceVoltage: 3;
  unsigned int sampleHoldTime: 4;
  unsigned int clockSource: 2;
  unsigned int clockDiv: 3;
  unsigned int sampleHoldSource: 2;
  unsigned int refVolt2_5: 1;
} __attribute__ ((packed)) MSP430ADC12AdvancedSettings_t;

typedef union {
   uint32_t i;
   MSP430ADC12AdvancedSettings_t s;
} MSP430ADC12AdvancedSettings_ut;

typedef enum {
   ADC_FAIL = 0,
   ADC_SUCCESS = 1,
   ADC_QUEUED = 2
} adcResult_t;

enum sampleHoldSource_enum { 
  HOLDSOURCE_TIMERA_OUT1 = 1,
  HOLDSOURCE_TIMERB_OUT0 = 2,
  HOLDSOURCE_TIMERB_OUT1 = 3
};

enum refVolt2_5_enum {
  REFVOLT_LEVEL_1_5 = 0,
  REFVOLT_LEVEL_2_5 = 1,
};
#define REFVOLT_LEVEL_NONE REFVOLT_LEVEL_1_5

enum clockDiv_enum {
   CLOCK_DIV_1 = 0,
   CLOCK_DIV_2 = 1,
   CLOCK_DIV_3 = 2,
   CLOCK_DIV_4 = 3,
   CLOCK_DIV_5 = 4,
   CLOCK_DIV_6 = 5,
   CLOCK_DIV_7 = 6,
   CLOCK_DIV_8 = 7,
};

enum inputChannel_enum {  
   INPUT_CHANNEL_A0 = 0,
   INPUT_CHANNEL_A1 = 1,
   INPUT_CHANNEL_A2 = 2,
   INPUT_CHANNEL_A3 = 3,
   INPUT_CHANNEL_A4 = 4,
   INPUT_CHANNEL_A5 = 5,
   INPUT_CHANNEL_A6 = 6,
   INPUT_CHANNEL_A7 = 7,
   EXTERNAL_REFERENCE_VOLTAGE = 8,
   REFERENCE_VOLTAGE_NEGATIVE_TERMINAL = 9,
   INTERNAL_TEMPERATURE = 10,
   INTERNAL_VOLTAGE = 11
};

enum referenceVoltage_enum {
   REFERENCE_AVcc_AVss = 0,
   REFERENCE_VREFplus_AVss = 1,
   REFERENCE_VeREFplus_AVss = 2,
   REFERENCE_AVcc_VREFnegterm = 4,
   REFERENCE_VREFplus_VREFnegterm = 5,
   REFERENCE_VeREFplus_VREFnegterm = 6
};

enum clockSource_enum {
   CLOCK_SOURCE_ADC12OSC = 0,
   CLOCK_SOURCE_ACLK = 1,
   CLOCK_SOURCE_MCLK = 2,
   CLOCK_SOURCE_SMCLK = 3
};

enum sampleHold_enum {
   SAMPLE_HOLD_4_CYCLES = 0,
   SAMPLE_HOLD_8_CYCLES = 1,
   SAMPLE_HOLD_16_CYCLES = 2,         
   SAMPLE_HOLD_32_CYCLES = 3,         
   SAMPLE_HOLD_64_CYCLES = 4,         
   SAMPLE_HOLD_96_CYCLES = 5,        
   SAMPLE_HOLD_123_CYCLES = 6,        
   SAMPLE_HOLD_192_CYCLES = 7,        
   SAMPLE_HOLD_256_CYCLES = 8,        
   SAMPLE_HOLD_384_CYCLES = 9,        
   SAMPLE_HOLD_512_CYCLES = 10,        
   SAMPLE_HOLD_768_CYCLES = 11,        
   SAMPLE_HOLD_1024_CYCLES = 12       
};

enum {
  TOSH_ACTUAL_ADC_EXTERNAL_REFERENCE_VOLTAGE_PORT = ASSOCIATE_ADC_CHANNEL( 
           EXTERNAL_REFERENCE_VOLTAGE, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5), 
  TOSH_ACTUAL_ADC_REFERENCE_VOLTAGE_NEGATIVE_TERMINAL_PORT = ASSOCIATE_ADC_CHANNEL( 
           REFERENCE_VOLTAGE_NEGATIVE_TERMINAL, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5), 
  TOSH_ACTUAL_ADC_INTERNAL_TEMPERATURE_PORT = ASSOCIATE_ADC_CHANNEL( 
           INTERNAL_TEMPERATURE, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5), 
  TOSH_ACTUAL_ADC_INTERNAL_VOLTAGE_PORT = ASSOCIATE_ADC_CHANNEL( 
           INTERNAL_VOLTAGE, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5), 
};  

#define ADC12CTL0_BASIC_SINGLE (ADC12CTL0_DEFAULT | ADC12ON)
#define ADC12CTL1_BASIC_SINGLE (ADC12CTL1_DEFAULT & ~(0x06))
#define ADC12CTL0_BASIC_SINGLE_REPEAT ((ADC12CTL0_DEFAULT | ADC12ON) | MSC)
#define ADC12CTL1_BASIC_SINGLE_REPEAT ((ADC12CTL1_DEFAULT & ~(0x02)) | (0x04))
#define ADC12CTL0_BASIC_SEQUENCE ((ADC12CTL0_DEFAULT | ADC12ON) | MSC)
#define ADC12CTL1_BASIC_SEQUENCE (((ADC12CTL1_DEFAULT & ~(0x06)) | 0x02) | MSC)
#define ADC12CTL0_BASIC_SEQUENCE_REPEAT ((ADC12CTL0_DEFAULT | ADC12ON) | MSC)
#define ADC12CTL1_BASIC_SEQUENCE_REPEAT (ADC12CTL1_DEFAULT | 0x06)
#define ADC12CTL0_ADVANCED_SINGLE_REPEAT ((ADC12CTL0_DEFAULT | ADC12ON) & ~(MSC))
#define ADC12CTL0_ADVANCED_SEQUENCE ((ADC12CTL0_DEFAULT | ADC12ON) & ~(MSC))

typedef enum {
  TIMERA_OUT1 = 1,
  TIMERB_OUT0 = 2,
  TIMERB_OUT1 = 3
} MSP430ADC12Timer;

typedef enum {
  SINGLE_CHANNEL_SINGLE_CONVERSION = 0,
  SEQUENCE_OF_CHANNELS = 1,
  REPEAT_SINGLE_CHANNEL = 2,
  REPEAT_SEQUENCE_OF_CHANNELS = 3,
  INTERNAL_CHANNEL = 4,
  ADVANCED_SEQUENCE_OF_CHANNELS = 5,
  ADVANCED_REPEAT_SINGLE_CHANNEL = 6,
  ADVANCED_REPEAT_SEQUENCE_OF_CHANNELS = 7,
  ADC_IDLE = 8
} MSP430ADC12ConversionMode_t;

typedef struct {
  volatile unsigned
  inch: 4,
  sref: 3,
  eos: 1;
} __attribute__ ((packed)) adc12memctl_t;

typedef struct {
  adc12memctl_t memctl;
  unsigned int sampleHoldTime: 4;
  unsigned int refVolt2_5: 1;
  unsigned int queued: 1;
  unsigned int gotRefVolt: 1;
  volatile unsigned int locked: 1;
} __attribute__ ((packed)) bSettings_t;

typedef struct {
  adc12memctl_t memctl;
  unsigned int sampleHoldTime: 4;
  unsigned int refVolt2_5: 1;
  unsigned int gotRefVolt: 1;
  unsigned int clockSource: 2;         
  unsigned int clockDiv: 3;            
  unsigned int sampleHoldSource: 2;    
  volatile unsigned int locked: 1;
} __attribute__ ((packed)) aSettings_t;

typedef struct {
  uint16_t *dataDest;
  uint16_t jiffies;
  uint8_t length;
  volatile uint8_t type;
  uint8_t intf;
} aBuffer_t;

#endif
