/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Marco Hutta

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004
*/

/**
 *
 * @author Marco Hutta
 *
 */

#ifndef _H_hardware_h
#define _H_hardware_h

#include "msp430hardware.h"
#include "MSP430ADC12.h"
#include "AM.h"

// LEDS
TOSH_ASSIGN_PIN(RED_LED, 2, 0);
TOSH_ASSIGN_PIN(GREEN_LED, 2, 1);
TOSH_ASSIGN_PIN(YELLOW_LED, 2, 2);

// RADIO
TOSH_ASSIGN_PIN(RFM_RXD,  3, 5);
TOSH_ASSIGN_PIN(RFM_TXD,  3, 4);
TOSH_ASSIGN_PIN(RFM_CTL0, 5, 6);
TOSH_ASSIGN_PIN(RFM_CTL1, 5, 7);

// UART
TOSH_ASSIGN_PIN(SOMI0, 3, 2);
TOSH_ASSIGN_PIN(SIMO0, 3, 1);
TOSH_ASSIGN_PIN(UCLK0, 3, 3);
TOSH_ASSIGN_PIN(UTXD0, 3, 4);
TOSH_ASSIGN_PIN(URXD0, 3, 5);
TOSH_ASSIGN_PIN(UTXD1, 3, 6);
TOSH_ASSIGN_PIN(URXD1, 3, 7);
TOSH_ASSIGN_PIN(UCLK1, 5, 3);
TOSH_ASSIGN_PIN(SOMI1, 5, 2);
TOSH_ASSIGN_PIN(SIMO1, 5, 1);

// ADC
TOSH_ASSIGN_PIN(ADC0, 6, 0); // microphone_in
TOSH_ASSIGN_PIN(ADC1, 6, 1); // pir_sensor_digital
TOSH_ASSIGN_PIN(ADC2, 6, 2); // pir_analog_output
TOSH_ASSIGN_PIN(ADC3, 6, 3); // external_power_supply
TOSH_ASSIGN_PIN(ADC4, 6, 4); // battery_power_supply
TOSH_ASSIGN_PIN(ADC5, 6, 5); // transceiver receive power
TOSH_ASSIGN_PIN(ADC6, 6, 6); // ir_light_power
TOSH_ASSIGN_PIN(ADC7, 6, 7); // nc

TOSH_ASSIGN_PIN(IR_SEND,1,0); 
TOSH_ASSIGN_PIN(IR_LIGHT,1,1);
TOSH_ASSIGN_PIN(IR_REC_IC,1,2);
TOSH_ASSIGN_PIN(PIR_SENSOR,1,3);
TOSH_ASSIGN_PIN(TILT,1,4);
TOSH_ASSIGN_PIN(CLOCK_ALARM,1,5);
TOSH_ASSIGN_PIN(POTI_INC,2,4);
TOSH_ASSIGN_PIN(POTI_UD,2,5);
TOSH_ASSIGN_PIN(POTI_CD,2,6);
TOSH_ASSIGN_PIN(PROG_TX,3,6);
TOSH_ASSIGN_PIN(PROG_RX,3,7);

// CON4
TOSH_ASSIGN_PIN(CON4_0,3,0);
TOSH_ASSIGN_PIN(CON4_1,3,1);
TOSH_ASSIGN_PIN(CON4_2,3,2);
TOSH_ASSIGN_PIN(CON4_3,3,3);

// CON5
TOSH_ASSIGN_PIN(CON5_0,4,0);
TOSH_ASSIGN_PIN(CON5_1,4,1);
TOSH_ASSIGN_PIN(CON5_2,4,2);
TOSH_ASSIGN_PIN(CON5_3,4,3);
TOSH_ASSIGN_PIN(CON5_4,4,4);
TOSH_ASSIGN_PIN(CON5_5,4,5);
TOSH_ASSIGN_PIN(CON5_6,4,6);
TOSH_ASSIGN_PIN(CON5_7,4,7);

void TOSH_SET_PIN_DIRECTIONS(void)
{
	P1SEL = 0x00;
	P1DIR = 0x81;
	P1OUT = 0x00;
	P2SEL = 0x00;
	P2DIR = 0x7F;
	P3OUT = 0x77;
	P3SEL = 0xE0;
	P3DIR = 0x5F;
	P3OUT = 0xE0;
	P4SEL = 0x00;
	P4DIR = 0xFF;
	P4OUT = 0x00;
	P5SEL = 0x00;
	P5DIR = 0xF0;
	P5OUT = 0x2F;
	P6SEL = 0x00;
	P6DIR = 0x00;
	P6OUT = 0x00;
}

enum {
  ECR,
  ESB,
  ESF,
};

enum {
	ESWBPS_2400,
	ESWBPS_19200,
	ESWBPS_38400,
	ESWBPS_57600,
	ESWBPS_115200,
};

#ifndef SENSORBOARD_H
#define SENSORBOARD_H

enum {
  TOSH_ADC_PORTMAPSIZE = 8 // default board has no sensors hooked up
};

enum
{
  TOSH_ACTUAL_ADC_PAR_PORT = ASSOCIATE_ADC_CHANNEL(
         INPUT_CHANNEL_A4, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5
  ),
  TOSH_ACTUAL_ADC_TSR_PORT = ASSOCIATE_ADC_CHANNEL(
         INPUT_CHANNEL_A5, REFERENCE_VREFplus_AVss, REFVOLT_LEVEL_1_5
  ),
};

enum
{
  TOS_ADC_PAR_PORT,
  TOS_ADC_TSR_PORT,
  TOS_ADC_INTERNAL_TEMP_PORT,
  TOS_ADC_INTERNAL_VOLTAGE_PORT
};

#endif

#endif

